package a.test.app;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletContext;
import java.io.IOException;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * Paul Hammant / ThoughtWorks / March 2009
 */
public class TestServlet extends HttpServlet {

    private static int ctr = 0;
    private int ctr2 = 0;
    private final long whenMade = System.currentTimeMillis();

    protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        ServletOutputStream os = response.getOutputStream();
        os.print("<html><body><p>");
        response.setContentType("text/html");
            HttpSession session = request.getSession(true);
            ServletContext ctx = session.getServletContext();
            Map ctxMap = (Map) ctx.getAttribute("ctx-map");
            if (ctxMap == null) {
                ctxMap = new HashMap();
                ctx.setAttribute("ctx-map", ctxMap);
            }

            AppScoped objOne = (AppScoped) ctxMap.get("ctx-obj");
            if (objOne == null) {
                objOne = new AppScoped();
                ctxMap.put("ctx-obj", objOne);
            }
            objOne.inc();

            Map sessMap = (Map) session.getAttribute("session-map");
            if (sessMap == null) {
                sessMap = new HashMap();
            }

            SessionScoped objTwo = (SessionScoped) sessMap.get("session-obj");
            if (objTwo == null) {
                objTwo = new SessionScoped(objOne);
                sessMap.put("session-obj", objTwo);
            }
            objTwo.inc();

            session.setAttribute("session-map", sessMap);

            os.println("virtual machine: instance-id=<span id=\"vm\" style=\"color:blue\">"
                    + System.identityHashCode(Runtime.getRuntime()) + "</span></br>");
            os.println("servlet: instance-id=<span style=\"color:green\">"
                    + System.identityHashCode(this) + "</span>, static-counter=" + ++ctr
                    + ", instance-counter=" + ++ctr2 + ", instance-age: "
                    + millisecondToDHMS(System.currentTimeMillis() - whenMade) + "</br>");
            os.println("session instance: instance-id=<span style=\"color:blue\">"
                    + System.identityHashCode(session) + "</span>, session-id: <span id=\"sess-id\" style=\"color:blue\">" + session.getId()
                    + "</span>, age:" + millisecondToDHMS(System.currentTimeMillis() - session.getCreationTime()) + "</br>");
            os.println("map in <u>servlet context attributes</u>: instance-id=<span style=\"color:blue\">"
                    + System.identityHashCode(ctxMap) + "</span></br>");
            os.println("App-Scoped object (#1) in that map that counts: instance-id=<span id=\"obj1\" style=\"color:blue\">" + objOne.identify()
                    + "</span>, count=<span id=\"ct1\" style=\"color:red\">" + objOne.val() + "</span>, object-age: "
                    + objOne.howOld() + "</br>");
            os.println("map in <u>session attributes</u>: instance-id=<span style=\"color:blue\">"
                    + System.identityHashCode(sessMap) + "</span>" + "</br>");
            os.println("Session-scoped object (#2) in that map that counts: instance-id=<span id=\"obj2\" style=\"color:blue\">" + objTwo.identify()
                    + "</span>, count=<span id=\"ct2\" style=\"color:red\">" + objTwo.val() + "</span>, object-age: " + objTwo.howOld()
                    + " it also has a dependency on (#1):"
                    + " dependant-instance-id=<span id=\"obj2dep\" style=\"color:blue\">" + objTwo.dependancysIdentify()+ "</span></br>");

            session.setMaxInactiveInterval(48 * 60 * 60);

            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new ServletException("sleep interrupted");
            }


        os.print("</p></body></html>");
    }

//     copied from web - http://google.com/search?q=millisecondToDHMS
    public final static long ONE_SECOND = 1000;
    public final static long ONE_MINUTE = ONE_SECOND * 60;
    public final static long ONE_HOUR = ONE_MINUTE * 60;
    public final static long ONE_DAY = ONE_HOUR * 24;

    public static String millisecondToDHMS(long duration) {
        String res = "";
        long temp = 0;
        if (duration >= ONE_SECOND) {
            temp = duration / ONE_DAY;
            if (temp > 0) {
                res = temp + " day";
                if (temp > 1) {
                    res += "s";
                }
                duration -= temp * ONE_DAY;

                if (duration >= ONE_MINUTE) {
                    res += ", ";
                }
            }

            temp = duration / ONE_HOUR;
            if (temp > 0) {
                res += temp + " hr";
                if (temp > 1) {
                    res += "s";
                }
                duration -= temp * ONE_HOUR;
                if (duration >= ONE_MINUTE) {
                    res += ", ";
                }
            }

            temp = duration / ONE_MINUTE;
            if (temp > 0) {
                res += temp + " min";
                if (temp > 1) {
                    res += "s";
                }
                duration -= temp * ONE_MINUTE;

                if (duration >= ONE_SECOND) {
                    res += " and ";
                }
            }

            temp = duration / ONE_SECOND;
            if (temp > 0) {
                res += temp + " sec";
                if (temp > 1) {
                    res += "s";
                }
            }
            return res;
        } else {
            return "0 second";
        }
    }

}
